# Django settings for example_project project.
import os

import dj_database_url


def project_dir(*paths):
    base = os.path.realpath(os.path.dirname(__file__))
    return os.path.join(base, *paths)


DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {'default': dj_database_url.config(default='sqlite:///' +
    project_dir('example_project.sqlite'))}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'


# Make this unique, and don't share it with anybody.
SECRET_KEY = 'lolimasekrit'

ROOT_URLCONF = 'example_project.urls'


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'example_project.polls',

    # my app
    'django_object_actions',

    # dev helpers
    'django_extensions',
    # testing
    'django_nose',
)

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'



from django.test import TestCase

from example_project.polls.models import Choice


class LoggedInTestCase(TestCase):
    def setUp(self):
        super(LoggedInTestCase, self).setUp()
        self.client.login(username='admin', password='admin')


class AppTests(LoggedInTestCase):
    def test_bare_mixin_works(self):
        # hit admin that doesn't have any tools defined, just the mixin
        response = self.client.get('/admin/polls/poll/add/')
        self.assertEqual(response.status_code, 200)

    def test_configured_mixin_works(self):
        # hit admin that does have any tools defined
        response = self.client.get('/admin/polls/choice/add/')
        self.assertEqual(response.status_code, 200)
        self.assertIn('objectactions', response.context_data)

    def test_tool_func_gets_executed(self):
        c = Choice.objects.get(pk=1)
        votes = c.votes
        response = self.client.get('/admin/polls/choice/1/tools/increment_vote/')
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['location'].endswith('/admin/polls/choice/1/'))
        c = Choice.objects.get(pk=1)
        self.assertEqual(c.votes, votes + 1)

    def test_tool_can_return_httpresponse(self):
        # we know this url works because of fixtures
        url = '/admin/polls/choice/2/tools/edit_poll/'
        response = self.client.get(url)
        # we expect a redirect
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['location'].endswith('/admin/polls/poll/1/'))

    def test_can_return_template(self):
        # This is more of a test of render_to_response than the app, but I think
        # it's good to document that this is something we can do.
        url = '/admin/polls/poll/1/tools/delete_all_choices/'
        response = self.client.get(url)
        self.assertTemplateUsed(response, "clear_choices.html")

    def test_intermediate_page_with_post_works(self):
        self.assertTrue(Choice.objects.filter(poll=1).count())
        url = '/admin/polls/poll/1/tools/delete_all_choices/'
        response = self.client.post(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Choice.objects.filter(poll=1).count(), 0)



from django.contrib import admin
from django.core.urlresolvers import reverse
from django.db.models import F
from django.http import HttpResponseRedirect

from django_object_actions import (DjangoObjectActions,
        takes_instance_or_queryset)

from .models import Choice, Poll


class ChoiceAdmin(DjangoObjectActions, admin.ModelAdmin):
    list_display = ('poll', 'choice_text', 'votes')

    @takes_instance_or_queryset
    def increment_vote(self, request, queryset):
        queryset.update(votes=F('votes') + 1)
    increment_vote.short_description = "+1"
    increment_vote.label = "vote++"

    def decrement_vote(self, request, obj):
        obj.votes -= 1
        obj.save()
    decrement_vote.short_description = "-1"

    def reset_vote(self, request, obj):
        obj.votes = 0
        obj.save()
    reset_vote.short_description = "0"

    def edit_poll(self, request, obj):
        url = reverse('admin:polls_poll_change', args=(obj.poll.pk,))
        return HttpResponseRedirect(url)

    objectactions = ('increment_vote', 'decrement_vote', 'reset_vote',
        'edit_poll')
    actions = ['increment_vote']

admin.site.register(Choice, ChoiceAdmin)


class ChoiceInline(admin.StackedInline):
    model = Choice
    extra = 3


class PollAdmin(DjangoObjectActions, admin.ModelAdmin):
    fieldsets = [
        (None,               {'fields': ['question']}),
        ('Date information', {'fields': ['pub_date'], 'classes': ['collapse']}),
    ]
    inlines = [ChoiceInline]
    list_display = ('question', 'pub_date', 'was_published_recently')
    list_filter = ['pub_date']
    search_fields = ['question']
    date_hierarchy = 'pub_date'

    def delete_all_choices(self, request, obj):
        from django.shortcuts import render_to_response
        from django.template import RequestContext

        if request.method == 'POST':
            obj.choice_set.all().delete()
            return

        return render_to_response('clear_choices.html',
            dict(object=obj), context_instance=RequestContext(request))
    delete_all_choices.label = "Delete All Choices"

    objectactions = ('delete_all_choices', )


admin.site.register(Poll, PollAdmin)



from django.db.models.query import QuerySet
from django.test import TestCase

from example_project.polls.models import Poll

from ..utils import QuerySetIsh, takes_instance_or_queryset


class QuerySetIshTest(TestCase):
    def setUp(self):
        # as defined in initial_data fixture
        # WISHLIST don't depend on fixture
        self.obj = Poll.objects.get(pk=1)

    def test_can_turn_object_into_queryset(self):
        qs = QuerySetIsh(self.obj)
        self.assertEqual(qs.count(), 1)
        self.assertEqual(qs.get(), self.obj)
        self.assertEqual(qs.order_by('foo').get(), self.obj)
        self.assertEqual(qs.all().get(), self.obj)
        self.assertEqual(qs.filter().get(), self.obj)
        self.assertEqual(qs.latest('bar'), self.obj)


class DecoratorTest(TestCase):
    def setUp(self):
        # as defined in initial_data fixture
        # WISHLIST don't depend on fixture
        self.obj = Poll.objects.get(pk=1)
        self.queryset = Poll.objects.all()

    def test_trivial(self):
        # setup
        def myfunc(foo, bar, queryset):
            return queryset

        # make sure my test function outputs the third arg
        self.assertEqual(myfunc(None, None, 'foo'), 'foo')
        # or the `queryset` kwarg
        self.assertEqual(myfunc(None, None, queryset='bar'), 'bar')

    def test_decorated(self):
        # setup
        @takes_instance_or_queryset
        def myfunc(foo, bar, queryset):
            return queryset

        # passing in an instance yields a queryset (using positional args)
        queryset = myfunc(None, None, self.obj)
        self.assertIsInstance(queryset, QuerySet)
        # the resulting queryset only has one item and it's self.obj
        self.assertEqual(queryset.get(), self.obj)

        # passing in a queryset yields the same queryset
        queryset = myfunc(None, None, self.queryset)
        self.assertIsInstance(queryset, QuerySet)
        self.assertEqual(queryset, self.queryset)

        # passing in an instance yields a queryset (using keyword args)
        queryset = myfunc(None, None, queryset=self.obj)
        self.assertIsInstance(queryset, QuerySet)
        # the resulting queryset only has one item and it's self.obj
        self.assertEqual(queryset.get(), self.obj)
